package.cpath = "..\\?.dll;..\\clib\\?.dll;"..package.cpath
package.path = "..\\?.lua;..\\lualib\\?.lua;"..package.path
local shared = assert( require 'shared.shared' )
local packet = shared.packet
local socket = assert( require 'socket' )
local mscxn = assert( socket.udp() )
mscxn:settimeout( 0 )
assert( mscxn:setpeername( shared.metaserver.ip, shared.metaserver.port ), "Could not connect to metaserver!" )
local udp
local io = assert( io )

local svInfo = packet.serverInfo{ version = 13,
  players = 0,
  capacity = 255,
  ip = shared.ip.fromString( socket.dns.toip(socket.dns.gethostname()) ),
  port = 51312,
  svname = "New Server",
  map = "Test Map"
}

local clients = {}
local connecting = {}
local server = { tick = 0, }

local handlers = setmetatable({

    clChimo = function( clChimo, ip, port )
      if not connecting[ ip..port ] then
        return print( "Old connection attempt from:", ip, port )
      end
      local remoteHash = clChimo.hash
      local clNonce = clChimo.nonce
      local svNonce = connecting[ip..port].nonce
      local localHash = shared.hash.hash( clNonce, svNonce )
      if localHash ~= remoteHash then
        return print( "Hashes differ:", shared.hash.hex( clNonce ), shared.hash.hex( svNonce ) )
      end
      print( "Client connected:", ip, port, localHash )
      clients[ localHash ] = { ip = ip, port = port }
      packet.connected{ token = localHash }
      return udp:sendto( packet.get(), ip, port )
    end,

    advertised = function( ack, ip, port )
      if ip ~= shared.metaserver.ip then return print( "Advertisement acked from rogue address:", ip, port ) end
      if udp:getsockname() then 
        assert( udp:getsockname() == tostring( ack.ip ) )
        return print( "Advertised. Address already set." )
      end
      print( "Advertised. Setting address:", ack.ip, ack.port )
      server.SetIP( tostring( ack.ip ), ack.port )
    end,


    clientInfo = function( clientInfo, ip, port )
      local key = ip..port 
      connecting[key] = connecting[key] or {}
      local client = connecting[key]
      local nonce = shared.hash.rand()
      client.nonce = nonce
      print( "Received connection request from:", ip, port )
      print( "Sending authentication nonce:", nonce )
      packet.svChimo{ nonce = nonce }
      return udp:sendto( packet.get(), ip, port )
    end,


    }, {__index = function() return print end })

function server.Advertise()
  print( "Advertise." )
  packet.get()
  packet.metaServer()
  packet.serverInfo( svInfo )
  mscxn:send( packet.get() )
end

function server.serverInfo( serverInfo )
  print( packet.getString( serverInfo.svname ) )
end

function server.metaServer( ms )
  server.advertising = true
end

--Incoming packet.
function server.Parse( msg, ip, port )
  if not msg then return end
  if not clients[ip] then
    print( "New IP:", ip, port )
    clients[ip] = { ip = ip, port = port }
  end

  local msgs, types = packet.deserialise( msg )
  if msgs then for i = 1, #msgs do
    print( "Received: ", types[i], ip, port )
    handlers[ types[i] ]( msgs[i], ip, port )
  end end
  return server.Parse( udp:receivefrom() ) -- Process other packets.
end

function server.SetIP( ipString, port )
  svInfo.ip = shared.ip.fromString( ipString )
  svInfo.port = port
  local ok, err = udp:setsockname( ipString, port )
  if ok then
    print( "Server IP:", udp:getsockname() )
    return true
    --Find another port.
  elseif port < 65536 then
    print( "Could not use port:", ipString, port, err )
    return server.SetIP( ipString, port + 1 )
  else
    print( "Could not use IP:", ipString, err )
    return error( "Connection failed." )
  end
end


function server.Start()
  udp = assert( socket.udp() )
  udp:settimeout(0)
end

function server.Advance()
  server.tick = server.tick + 1
  for id, client in pairs( clients ) do
    --
  end
end

function server.NewGame()
  server.tick = 0
end

function server.Quit()

end

server.Start()

function love.update( dt )
  server.Parse( udp:receivefrom() )
  server.Advance()
  if server.tick % 250 == 0 then
    server.Advertise()
    server.Parse( mscxn:receive(), shared.metaserver.ip, shared.metaserver.port )
  end
end

function love.draw() end